/*
 * Code Mercenaries Hard- und Software GmbH
 * Karl-Marx-Strasse 147A
 * 12529 Schönefeld
 * Deutschland
 * 
 * August 2015
 * 
 * Used result values:
 *   -1 hard error
 * 	  1 light weight error (like wrong user input)
 *    0 success
 * 
 * This program is using the BCM2835 library for 
 * Raspberry Pi 2. More information at
 * http://www.airspayce.com/mikem/bcm2835/
 * */
 
#include <stdio.h>
#include <string.h>
#include <bcm2835.h>
#include <errno.h>
#include <stdlib.h>
#include "led_warrior09.h"

void printHelp();
char printStatus();
char ReadArguments(int, char**);
void printConfig();

/*
 * Read in the command line arguments
 * */
char readArguments(int argc, char** argv) {
	
    int buf;
    char mode;
    char result;
	
	//search for help command
    for(int i = 1; i<argc; i++) {
        
        if(strstr(argv[i], "help")) {
            printHelp();
            return 0;
        }
    }
    //started program without parameter
    if(argc == 1) {
        printHelp();
        return 0;
    }
    
    if((argc >= 2) && strcmp(argv[1], "-l") == 0) {
        result = LW09_listDevices();
        if(result != 0) return -1;
        return 0;
    }
    
    //get address for the I2C device
    if((argc >= 3) && (strcmp(argv[1], "-a") == 0)){
        buf = atoi(argv[2]);
        if(buf >= 0 && buf <= 127) {
            LW09_setSlave(buf);
        }else {
            printf("Invalid address! Address has to be between 0 and 127\r\n");
            return 1;
        }
    }else {
        printf("Destination address not given!\r\n");
        return 1;
    }
	
    //i = 3 to start by "real" parameter (argv[0] is the command for starting this program and -a is obligatory)
	for(int i = 3; i<argc; i++) {
		
		if(argv[i][0] == '-' && strlen(argv[i])>1) {
			
			//get parameter
			switch(argv[i][1]) {
                           
                //set Command
                case 'd':	if(i+4>=argc) return 1;
							result = LW09_setCmd(atoi(argv[i+1]),
												 atoi(argv[i+2]),
												 atoi(argv[i+3]),
											     atoi(argv[i+4]));
							i = i+4;
							break;
                
                //set (W) or get (R) config
                case 'g':	if(i+1 >= argc) return 1;
							
							//first argument
                            switch(argv[++i][0]) {
                                
                                case 'W':   mode = LW09_WRITE;
                                            break;
                                            
                                case 'R':   mode = LW09_READ;
                                            break;
                                            
                                default:    return 1;
                            }
                            
                            //write needs 4 additional arguments
                            if(mode == LW09_WRITE && i+4 >= argc) return 1;
                            
                            //do work
							if(mode == LW09_WRITE) {
								result = LW09_setConfig(atoi(argv[i+1]),
														atoi(argv[i+2]),
														atoi(argv[i+3]),
														atoi(argv[i+4]));
								
								i = i+4;
								
							}else if(mode == LW09_READ) {
								printConfig();
							}
                            
							break;
                
                //change I2C address
                case 'i':	if(++i >= argc) return 1;
							result = LW09_changeToAddressI2C(atoi(argv[i]));
							break;
                
                //get status
                case 's':	result = printStatus();
							break;
                            
                //list all possible devices with addresses
                case 'l':   result = LW09_listDevices();
                            break;
                            
                //no valid parameter, show help
				default:	printHelp();
                            return 0;
                            
			}
			if(result != 0) return -1;
			
		}else {
            return 1;
        }
        
	}
    
	printf("Command accepted\r\n");
	
	return 0;
}

void printHelp() {
	
	printf("Usage:\r\n");
    printf("-l List all active device addresses. Has to be first argument and all other arguments are ignored\r\n\r\n");
    printf("-a x Has to be the first parameter if not using \"-l\" (x from 0 to 127)\r\n\r\n");
    printf("-d addr cmd y s\r\n");
    printf("   (check documentation of LED Warrior09 (chapter 4.3.1) for more information)\r\n\r\n");
    printf("-s Print status\r\n\r\n");
    printf("-i x Set the I2C device address to address x (x from 0 to 127)\r\n\r\n");
    printf("-g W mode_0 addr_0 mode_1 addr_1 Set config\r\n");
    printf("     (check documentation of LED Warrior09 (chapter 4.4) for more information)\r\n\r\n");
    printf("   R Print config\r\n\r\n");
}

char printStatus() {
	
	char status;
	char result = LW09_getStatus(&status);
	if(result != 0) return -1;
	
    if((status & 0x80) == 0x80) {
		printf("Status: Bus fault\r\n");
	}else {
		printf("Status: Bus OK\r\n");
	}
	
	if((status & 0x40) == 0x40) {
		printf("Status: busy\r\n");
	}else {
		printf("Status: ready\r\n");
	}
	
	return 0;
}

void printConfig() {
	
    char buffer[4] = {0};
    LW09_getConfig(buffer);
    
    printf("mode 0: %02x\r\n", buffer[0]);
    printf("address 0: %02x\r\n", buffer[1]);
    printf("mode 1: %02x\r\n", buffer[2]);
    printf("address 1: %02x\r\n", buffer[3]);
}

int main(int argc, char** argv) {
	
	//initialize library for I2C
	if(!bcm2835_init()) {
		printf("No initialization possible!\r\n");
		return 1;
    }
    //begin connection
    bcm2835_i2c_begin();
    bcm2835_i2c_setSlaveAddress(0);
    bcm2835_i2c_setClockDivider(BCM2835_I2C_CLOCK_DIVIDER_2500);

	signed char result = readArguments(argc, argv);
    
    if(result == 1) {
		printf("Error by reading in the arguments!\r\n");
		return 1;
		
	}else if(result == -1) {
		printf("Internal error\r\n");
		return 1;
	}
    
    //end connection
    bcm2835_i2c_end();
    
    return 0;
}
