/*
 * Code Mercenaries Hard- und Software GmbH
 * Karl-Marx-Strasse 147A
 * 12529 Schönefeld
 * Deutschland
 * 
 * August 2015
 * 
 * Used result values:
 *   -1 hard error
 * 	  1 light weight error (like wrong user input)
 *    0 success
 * 
 * This program is using the BCM2835 library for 
 * Raspberry Pi 2. More information at
 * http://www.airspayce.com/mikem/bcm2835/
 * */
 
//register
#define LW09_REG_STATUS 0x00
#define LW09_REG_CMD    0x01
#define LW09_REG_CONFIG 0x08
#define LW09_REG_ADDR   0xFE

//command codes
#define LW09_CC_OFF                  0x00
#define LW09_CC_UP                   0x01
#define LW09_CC_DOWN                 0x02
#define LW09_CC_STEP_UP              0x03
#define LW09_CC_STEP_DOWN            0x04
#define LW09_CC_MAX_LEVEL            0x05
#define LW09_CC_MIN_LEVEL            0x06
#define LW09_CC_STEP_DOWN_AND_OFF    0x07
#define LW09_CC_ON_AND_STEP_UP       0x08
#define LW09_CC_ENABLE_DAPC_SEQUENCE 0x09
#define LW09_CC_GO_TO_SCENE          10 //possible values from 0x10 to 0x1F

#define LW09_WRITE 0
#define LW09_READ  1

//include read and write methods with error check
#include "errorCheckedBCM2835.h"

char LW09_setStartregister(int);

char LW09_getStatus();

char LW09_setCmd(char, char, char, char);

char LW09_getConfig(char*);
char LW09_setConfig(char, char ,char, char);

char LW09_changeToAddressI2C(char);
char LW09_listDevices();

/*
 * Set the address we are reading from and
 * writing to
 * */
void LW09_setSlave(int address) {
    
    bcm2835_i2c_setSlaveAddress(address);
}

/*
 * Set intern device pointer to reg
 * */
char LW09_setStartregister(int reg) {
	
	char buffer[1];//only set register, not write data
	buffer[0] = reg;
	//write one byte with the destination register
	char result = errorCheck_bcm2835_i2c_write(buffer, 1);
	if(result != 0) 
		return -1;
	
	return 0;
}

/*
 * Read status of device
 * */
char LW09_getStatus(char* dest) {
	
	char buffer[1] = {0};
	
	//set pointer to reg for reading task
	char result = LW09_setStartregister(LW09_REG_STATUS);
	if(result != 0) 
		return -1;
	
	//read 2 bytes in to buffer
	result = errorCheck_bcm2835_i2c_read(buffer, 1);
	if(result != 0) 
		return -1;
	
	*dest = buffer[0];
	
	return 0;
}

/*
 * Set command
 * */
char LW09_setCmd(char addr, char cmd, char y, char s) {
	
	//address = 64, shift left one
	char buffer[3];
	buffer[0] = LW09_REG_CMD;
	buffer[1] = (addr << 1) | ((y <<7) & 0x80) | (s & 0x01);
	buffer[2] = cmd;
	
	//write 3 bytes
	char result = errorCheck_bcm2835_i2c_write(buffer, 3);
	if(result != 0) 
		return -1;
	
	return 0;
}

/*
 * get information from config register
 * */
char LW09_getConfig(char* buffer) {
	
	//set pointer to reg for reading task
	char result = LW09_setStartregister(LW09_REG_CONFIG);
	if(result != 0) 
		return -1;
	
	//read 4 bytes
	result = errorCheck_bcm2835_i2c_read(buffer, 4);
	if(result != 0) 
		return -1;
	
	return 0;
}

/*
 * set values to config register
 * */
char LW09_setConfig(char mode_0, char addr_0 ,char mode_1, char addr_1) {
	
	char buffer[5];
	buffer[0] = LW09_REG_CONFIG;
	buffer[1] = mode_0;
	buffer[2] = addr_0;
	buffer[3] = mode_1;
	buffer[4] = addr_1;
	
	//write 5 bytes
	char result = errorCheck_bcm2835_i2c_write(buffer, 5);
	if(result != 0) 
		return -1;
	
	return 0;
}

/*
 * Change I2C address permanent for this device
 * */
char LW09_changeToAddressI2C(char addr) {
	
	char buffer[3];
	buffer[0] = LW09_REG_ADDR;
	buffer[1] = addr;
	//negated addr value to check by the chip
	buffer[2] = ~addr;
	
	//write 3 bytes
	char result = errorCheck_bcm2835_i2c_write(buffer, 3);
	if(result != 0) 
		return -1;
	
	return 0;
}

/*
 * Get a list with all active devices on I2C bus
 * */
char LW09_listDevices() {
	
	//in this method we are using the standard read and write method,
	//so we dont get a possible NACK for every address
	
	char bufferRead[1] = {0};
	char result = 0;
	
	//scan through all device addresses (0 is broadcast)
	for(int i = 1; i<127; i++) {
		
		//clear buffer
		memset(bufferRead, 0, 1);
		
		//change address
		LW09_setSlave(i);
		
		//set pointer to reg for reading task
		char bufferWrite[1];//only set register, not write data
		bufferRead[0] = LW09_REG_STATUS;
		
		//write one byte with the destination register
		result = bcm2835_i2c_write(bufferWrite, 1);		
		if(result != BCM2835_I2C_REASON_OK) 
			continue;
		
		result = bcm2835_i2c_read(bufferRead, 1);
		if(result != BCM2835_I2C_REASON_OK) 
			continue;
		
		printf("Device found on address %d\r\n", i);
	}
	
	return 0;
}
